# $Id: Handler.pm 1820 2009-09-28 18:55:19Z wsl $
# $URL: http://svn.zo.oi/wsl/src/libxyzzy-perl/lib/Xyzzy/Handler.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Handler;

use Apache2::Request;
use Apache2::Const;
use Apache2::RequestIO ();

use Spiffy -Base;

our @EXPORT_BASE = qw(handler);

sub suite { 'Xyzzy' }
sub static { $self->suite . '::Static' }
sub config { $self->suite . '::Configuration' }
sub generators { undef }

sub handler: method {
	my $self = shift;
	my ($rec) = @_;

	$rec->filename('-');

	my $path = $rec->uri;

	my @fullpath = grep { $_ ne '' } split('/', $rec->uri);
	my @path = @fullpath;

	my $generator = $self->generators;
	while(ref $generator) {
		$generator = $generator->{shift @path or ''}
	}

	$generator = $self->static
		unless defined $generator;

	return Apache2::Const::NOT_FOUND
		unless defined $generator;

	my ($content, $code, $type);
	eval {
		my $page = $generator->generate($rec, $self->config, \@path, \@fullpath);
		$content = $page->content;
		$code = $page->code;
		$type = $page->type;
	};
	if($@) {
		warn "$generator: $@";
		return Apache2::Const::SERVER_ERROR;
	}

	if(defined $type) {
		$rec->content_type($type);
	} else {
		$rec->content_type('text/html');
	}
	$rec->headers_out->add('Pragma' => 'no-cache');
	$rec->headers_out->add('Cache-Control' => 'no-store');
	$rec->headers_out->add('Expires' => 'Thu, 01 Jan 1970 00:00:00 GMT');

	if(defined $content && !defined $code) {
		$rec->print($content);
		$code = Apache2::Const::OK;
	} else {
		$code = Apache2::Const::SERVER_ERROR
			unless defined $code;
	}
	return $code;
}
